package com.bomostory.sceneeditmodule.screen.movie

import android.graphics.Bitmap
import com.bomostory.sceneeditmodule.SceneDrawer
import java.io.File
import java.io.IOException

import org.jcodec.api.transcode.PixelStore
import org.jcodec.api.transcode.PixelStoreImpl
import org.jcodec.api.transcode.Sink
import org.jcodec.api.transcode.SinkImpl
import org.jcodec.api.transcode.VideoFrameWithPacket
import org.jcodec.common.AndroidUtil
import org.jcodec.common.Codec
import org.jcodec.common.Format
import org.jcodec.common.io.NIOUtils
import org.jcodec.common.io.SeekableByteChannel
import org.jcodec.common.model.ColorSpace
import org.jcodec.common.model.Packet
import org.jcodec.common.model.Picture
import org.jcodec.scale.ColorUtil
import org.jcodec.scale.Transform

class JcodecMovieEncoder: MovieEncoder {

    companion object {
        private val OUTPUT_FORMAT = Format.MOV
        private val OUTPUT_VIDEO_CODEC = Codec.H264
        private val OUTPUT_AUDIO_CODEC: Codec? = null
    }

    private var transform: Transform? = null
    private var sink: Sink? = null
    private var pixelStore: PixelStore? = null

    private var videoWidth = 1440
    private var videoHeight = 720
    private var fps: Int = 15

    private var frameNo = 1L
    private var timestamp = 0L

    constructor() {
    }

    override fun setVideoDimension(width: Int, height: Int) {
        videoWidth = width
        videoHeight = height
    }

    override fun prepare(file: File, fps: Int) {
        this.fps = fps

        try {
            val out = NIOUtils.writableChannel(file)
            sink = SinkImpl.createWithStream(out, OUTPUT_FORMAT, OUTPUT_VIDEO_CODEC, OUTPUT_AUDIO_CODEC)
            sink?.init()

            if (sink!!.inputColor != null)
                transform = ColorUtil.getTransform(ColorSpace.RGB, sink!!.inputColor)

            pixelStore = PixelStoreImpl()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    override fun start() {
        frameNo = 1L
        timestamp = 0L
    }

    override fun addFrame(bitmap: Bitmap, repeat: Long) {
        val duration = repeat
        var input = bitmap
        if (input.width != videoWidth || input.height != videoHeight) {
            input = Bitmap.createScaledBitmap(bitmap, videoWidth, videoHeight, true)
        }
        val picture = AndroidUtil.fromBitmap(input, ColorSpace.RGB)
        try {
            val sinkColor = sink?.inputColor
            val toEncode: PixelStore.LoanerPicture
            if (sinkColor != null) {
                toEncode = pixelStore!!.getPicture(videoWidth, videoHeight, sinkColor)
                transform!!.transform(picture, toEncode.picture)
            } else {
                toEncode = PixelStore.LoanerPicture(picture, 0)
            }

            val pkt = Packet.createPacket(null, timestamp, fps, duration, frameNo++, Packet.FrameType.KEY, null)
            sink?.outputVideoFrame(VideoFrameWithPacket(pkt, toEncode))

            if (sinkColor != null)
                pixelStore?.putBack(toEncode)

            timestamp += duration
        } catch (e: IOException) {
            e.printStackTrace()
        } catch (e: IllegalStateException) {
            e.printStackTrace()
        }
    }

    override fun finish() {
        try {
            sink?.finish()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }
}
