package com.bomostory.sceneeditmodule.utils

import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Canvas
import android.support.v4.content.FileProvider
import android.widget.Toast
import com.bomostory.sceneeditmodule.cover.AlignStyle
import com.example.tfat.myapplication.R
import java.io.File

object Utils {

    fun Bitmap.crop(alignStyle: AlignStyle): Bitmap {
        val src = this
        val size = Math.min(src.width, src.height)
        val dst = Bitmap.createBitmap(size, size, src.config)
        val canvas = Canvas(dst)
        val left = when (alignStyle) {
            AlignStyle.Left -> 0
            AlignStyle.Center -> -(src.width - dst.width)
            AlignStyle.Right -> -(src.width - dst.width) / 2
        }
        canvas.drawBitmap(src, left.toFloat(), 0f, null)
        return dst
    }

    fun generateSceneName(index: Int): String {
        val book = ('A'.toInt() + ((index) / 12)).toChar()
        val num = (index) % 12 + 1
        return "$book$num"
    }

    fun showExportPdfCompleteThenViewDialog(context: Context, authority: String, file: File) {
        AlertDialog.Builder(context)
                .setMessage(R.string.export_pdf_complete_dialog_msg)
                .setPositiveButton(R.string.export_pdf_complete_dialog_positive) { _, _ ->
                    val uri = FileProvider.getUriForFile(context, authority, file)
                    val intent = Intent(Intent.ACTION_VIEW, uri).apply {
                        addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                        addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY)
                        addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
                        setDataAndType(uri, "application/pdf")
                    }
                    if (intent.resolveActivity(context.packageManager) != null) {
                        Intent.createChooser(intent, "").apply {
                            context.startActivity(this)
                        }
                    } else {
                        Toast.makeText(context, R.string.share_pdf_failed_no_app, Toast.LENGTH_SHORT).show()
                    }
                }.show()
    }
}